#include "commandwidget.h"
#include <QPushButton>
#include <QTimer>
#include <QHBoxLayout>
#include <QMessageBox>
#include <QFileDialog>
#include <math.h>
#include <stdio.h>

// Application fabriquant le widget affiché au centre de la fenêtre principale cinesim
// qui affiche les boutons Play/pause, Un_pas-dir/un_pas_inv, Inverse, Init et
// un curseur pour faire évoluer le temps à la main.
// Cette application gère l'avancement du temps en fonction de l'appui sur ses boutons.

CommandWidget::CommandWidget(int dt_ms, int nbTics, QWidget *parent) : QWidget(parent)
{
    setWindowTitle(QObject::tr("CommandDialog"));
    setFocusPolicy( Qt::StrongFocus);

    //Les boutons et sliders
    labelSlider = new QLabel(tr("000000/000000"));
    labelSlider->setMinimumSize(labelSlider->sizeHint());
    slider = new QSlider(Qt::Horizontal);
    recButton = new QPushButton("REC");
    play_pauseButton = new QPushButton("PLAY");
    stepButton = new QPushButton("UN_PAS_DIR");
    invButton = new QPushButton("INVERSER");
    initButton = new QPushButton("INIT");

    // Le timer
    timer = new QTimer(this);
    timer->stop();
    modePlay = PAUSE; // On est en PAUSE

    // Seulement un HBoxLayout
    QHBoxLayout *mainLayout = new QHBoxLayout(this);
    mainLayout->addWidget(recButton);
    mainLayout->addWidget(play_pauseButton);
    mainLayout->addWidget(stepButton);
    mainLayout->addWidget(invButton);
    mainLayout->addWidget(initButton);
    mainLayout->addWidget(labelSlider);
    mainLayout->addWidget(slider);

    connect(play_pauseButton,SIGNAL(clicked()),this,SLOT(playpauseSlot()));
    connect(stepButton,SIGNAL(clicked()),this,SLOT(timeOutSlot()));
    connect(invButton,SIGNAL(clicked()),this,SLOT(invSlot()));
    connect(initButton,SIGNAL(clicked()),this,SLOT(initButtonSlot()));
    connect(slider,SIGNAL(valueChanged(int)),this,SLOT(setNechSlot(int)));
    connect(timer, SIGNAL(timeout()), this, SLOT(timeOutSlot()));


    // Le slider est utilisé comme synchronisateur unique de tous les éléments
    // qui agissent sur le temps : Boutons Init, Step, Lecture de conditions exterieures....
    // Tous ces éléments mettent à jour le slider avec un numero d'échantillon nech
    // par un slider->setValue(nech). L'usager en bougeant le slider, le modifie également.
    // Les modifications de valeur du slider effectuées par l'usager ou par un slider->setValue(nech)
    // sont répercutées sur le slot setNechSlot(int) qui met à jour nech, le transforme en
    // temps par nech2time et met à jour les visus.

    // Initialisation espace-temps
    sensTime = 1;
    setTimer(dt_ms);
    setSlider(0, nbTics);
}


// Reception clic sur bouton inversion sens de marche
void CommandWidget::invSlot()
{
    sensTime = -sensTime;
    if (sensTime > 0) stepButton->setText("UN_PAS_DIR");
    else stepButton->setText("UN_PAS_INV");
}

// Reception d'un tic horloge : On fait avencer le numero d'échantillon
void CommandWidget::timeOutSlot()
{
        nech += sensTime;
        if (nech == nechtot) // Si arrive a la fin
        {
            // On passe en pause et on remet le compteur d'echantillon a zero.
            playpauseSlot();
            nech = 0;
        }
        else if (nech == 0) // Si retour au debut
        {
            // On passe en pause et on remet le compteur d'echantillon a zero.
            playpauseSlot();
            nech = nechtot-1;
        }
        // Le slider se charge de la synchro du reste
        slider->setValue(nech); // Va produire setNechSlot
}

// Modification valeurs slider
void CommandWidget::setSlider(int numTic, int nbTics)
{
char ligne[24];

    nech = numTic;
    nechtot = nbTics;
    slider->setMaximum(nechtot-1);
    // Mise a jour format d'impression utilisé pour le label d'impression " nech / nechtot "
    int n = 1 + (int) log10(nechtot-1);
    sprintf(formdec, "%%0%dd/%%0%dd", n, n);
    sprintf(ligne,formdec, nech, nechtot-1);
    labelSlider->setText(ligne);
}

// Reception clic sur bouton Play/Pause
void CommandWidget::playpauseSlot()
{
    if (modePlay == PLAY) // On est en play et on a appuye sur pause
    {
        // On passe en pause
        timer->stop();
        modePlay = PAUSE;
        play_pauseButton->setText("PLAY");// On met le texte pour pouvoir repasser en Play
    }
    else if (modePlay == PAUSE)// On est en pause et on a appuye sur play
    {
        // On passe en play
        modePlay = PLAY;
        play_pauseButton->setText("PAUSE");// On met le texte pour pouvoir repasser en Play
        timer->start(dtime_ms);
    }
}

// Reception clic sur bouton INIT
void CommandWidget::initButtonSlot()
{
char ligne[24];
    sensTime = 1;
    nech = 0,
    slider->setValue(0);
    sprintf(ligne,formdec, nech, nechtot-1);
    labelSlider->setText(ligne);
    emit isTime(nech);
}

// Reception nech suite a une modification du slider.
void CommandWidget::setNechSlot(int m)
{
char ligne[24];
    nech = m;
    sprintf(ligne,formdec, nech, nechtot-1);
    labelSlider->setText(ligne);
    emit isTime(nech);
}
