#include "confdialog.h"
#include "ui_confdialog.h"
#include "../cinespace.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QModelIndexList>
#include <math.h>
#include "terre.h"
#include "starswidget.h"
#include "TwoLE.h"
#include <math.h>

#define toRadians (M_PI/180.)
#define toDegrees (180./M_PI)

// Les slider longitude et latitude sont en entier en centièmes de degré d'ou des
// Les doubleSpinBox sont en double en degrés. Ce sont eux qui avertissent CineSpace si modifiés
// conversions entier -> double, double entier

ConfDialog::ConfDialog(CineSpace *p, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ConfDialog), notinitok(true)
{
    ui->setupUi(this);
    cine = p;
    setWindowTitle("Configuration générale");

    initLongiLati(Terre::longobs, Terre::latobs);

    connect(ui->doubleSpinBox_lon, SIGNAL(valueChanged(double)),this, SLOT(slider_lon_setValue(double)));
    connect(ui->doubleSpinBox_lat, SIGNAL(valueChanged(double)),this, SLOT(slider_lat_setValue(double)));

    connect(ui->horizontalSlider_lon, SIGNAL(valueChanged(int)),this, SLOT(spinbox_lon_setValue(int)));
    connect(ui->horizontalSlider_lat, SIGNAL(valueChanged(int)),this, SLOT(spinbox_lat_setValue(int)));

    // Autorise la sélection multiple dans le ListView des satellites_TLE
    ui->listView_AddSats->setSelectionMode(QAbstractItemView::ExtendedSelection);
    // Le modèle de liste des satellites_TLE qui sont mis dans le ListView
    model_sattle = new QStringListModel(this);

    // Autorise la sélection multiple dans le ListView des satellites sélectionnés
    ui->listView_selSats->setSelectionMode(QAbstractItemView::ExtendedSelection);
    // Le modèle de liste des satellites selectionnés qui sont mis dans le ListView
    model_satsel = new QStringListModel(this);

    // Met à jour la précision
    ui->labelPrecision->setText(QString::number(cine->precision,'e', 2));
    int n = -10.*(3. + log10(cine->precision)) + 0.5;
    ui->horizontalSliderPrecision->setValue(n);

    notinitok = false;



}

ConfDialog::~ConfDialog()
{
    delete ui;
}

void  ConfDialog::initLongiLati(double longitude, double latitude)
{
    ui->doubleSpinBox_lon->setValue(longitude);
    ui->horizontalSlider_lon->setValue((int) (100*toDegrees * longitude));

    ui->doubleSpinBox_lat->setValue(toDegrees * latitude);
    ui->horizontalSlider_lat->setValue((int) (100.*toDegrees * latitude));
}

////////////////////////////////////////////////////////////////////////////////
// SLOTS de CONVERSION double -> int pour les connections doublespinbox -> slider
////////////////////////////////////////////////////////////////////////////////
void  ConfDialog::slider_lat_setValue(double v)
{ui->horizontalSlider_lat->setValue((int) rint(100.*v)); }

void  ConfDialog::slider_lon_setValue(double v)
{ui->horizontalSlider_lon->setValue((int) rint(100.*v)); }

/////////////////////////////////////////////////////////////////////////////////
// SLOTS de CONVERSION int -> double pour les connections slider -> doublespinbox
/////////////////////////////////////////////////////////////////////////////////
void  ConfDialog::spinbox_lat_setValue(int v)
{ui->doubleSpinBox_lat->setValue(((double) v)/100.);}

void  ConfDialog::spinbox_lon_setValue(int v)
{ui->doubleSpinBox_lon->setValue(((double) v)/100.);}

////////////////////////////////////
// Ajout de TLE lues dans un fichier
////////////////////////////////////
void ConfDialog::on_pushButton_addTLE_clicked()
{
    if (notinitok) return;
    QString NomFich = QFileDialog::getOpenFileName (NULL, QObject::tr("Fichier TLE ?"),".");
    if (NomFich.endsWith("txt",Qt::CaseInsensitive) || NomFich.endsWith("tle",Qt::CaseInsensitive))
    {
        // Lecture et ajout des satellites lus dans la TLE à la liste cine->all_tle
        int nb = TwoLE::readFile(NomFich.toLocal8Bit().data(), cine->all_tle);
        if (nb > 0)
        {
            QMessageBox::information(this,"AJOUT TLEs",tr("%1 TLE ajoutées, soit %2 tle en tout.").arg(nb).arg(cine->all_tle->size()));
            // On fait une nouvelle liste des noms des satellites lus
            QStringList l;
            for (int i = 0; i < cine->all_tle->size(); i++) l.append(cine->all_tle->at(i)->name);

            model_sattle->setStringList(l);
            ui->listView_AddSats->setModel(model_sattle);
        }
    }
    else QMessageBox::warning(this,"AJOUT TLE","L'extension du fichier doit être en *.txt ou *.tle !!");

}

///////////////////////////////////////////
// Ajout d'un satellite lue dans un fichier
///////////////////////////////////////////
void ConfDialog::on_pushButton_addSatFile_clicked()
{
    if (notinitok) return;
    QString NomFich = QFileDialog::getOpenFileName (NULL, QObject::tr("Fichier Satellite ?"),".");
    if (NomFich.endsWith("dat",Qt::CaseInsensitive))
    {

        SatGeo *sat = new SatGeo();
        int ok = sat->set(NomFich.toLocal8Bit().data());
        if(ok)
        {
            cine->all_sats->push_back(sat);
            // On fait une nouvelle liste des noms des satellites selectionnés
            makeListViewSatsSel();
        }
    }
}

/////////////////////////////////////////////////
// Ajout de satellittes depuis la liste des TLE
/////////////////////////////////////////////////
void ConfDialog::on_pushButton_AddSatsfromTle_clicked()
{
    if (notinitok) return;
    // Pour récupérer les index selectionnés
    QItemSelectionModel *selmodel = ui->listView_AddSats->selectionModel();
    if (selmodel == NULL) return;
    QModelIndexList indexes = selmodel->selectedIndexes();
    for (int k = 0 ; k < indexes.size() ; k++)
    {
        int i = indexes.at(k).row();
        SatGeo *sat = new SatGeo(cine->all_tle->at(i));
    // On l'ajoute en tête du vecteur des satellites connus
        vector<SatGeo *>::iterator iv = cine->all_sats->begin();
        cine->all_sats->insert(iv, sat);
    }
    makeListViewSatsSel();
}

void ConfDialog::makeListViewSatsSel()
{
    // On refait le listView des satellites selectionnés
        QStringList l;
        for (int i = 0; i < cine->all_sats->size(); i++) l.append(cine->all_sats->at(i)->nom);
        model_satsel->setStringList(l);
        ui->listView_selSats->setModel(model_satsel);
}


/////////////////////////////////////////////////
// Retranche des satellittes dans la sélection
////////////////////////////////////////////////
void ConfDialog::on_pusButton_OterSats_clicked()
{
    if (notinitok) return;
    // Pour récupérer les index selectionnés
    QItemSelectionModel *selmodel = ui->listView_selSats->selectionModel();
    if (selmodel == NULL) return;
    QModelIndexList indexes = selmodel->selectedIndexes();
    // On fait une nouvelle liste avec les satellites non sélectionnées
    vector<SatGeo *> *newsats = new vector<SatGeo *>();
    for (int i = 0 ; i < cine->all_sats->size() ; i++)
    {
        bool ok = true;
        for (int k = 0 ; k < indexes.size() ; k++)
        {
            if (i == indexes.at(k).row()) {ok = false; break;}
        }
        if (ok) newsats->push_back(cine->all_sats->at(i));
    }
    delete cine->all_sats;
    cine->all_sats = newsats;
    makeListViewSatsSel();
    // En cas de disparition du satellite courant, on sélectionne le premier de la liste
    cine->satcou = 0;
}


////////////////////////////////////
// Selection satellite courant
////////////////////////////////////
void ConfDialog::on_pushButton_selSatcou_clicked()
{
    if (notinitok) return;
//    cine->satcou = ui->comboBox_selSatcou->currentIndex();
    // Pour récupérer les index selectionnés car possibilité de sélection multiple
    QItemSelectionModel *selmodel = ui->listView_selSats->selectionModel();
    if (selmodel == NULL) return;
    QModelIndexList indexes = selmodel->selectedIndexes();
    if (indexes.size() == 1) cine->satcou = indexes.at(0).row();
    else {
        QMessageBox::warning(this,"Warning ConfDialog","Sélectionnez un seul satellite dans la liste !");
        return;
    }
}

///////////////////////////////////////////////////////////
// Si changement doubleSpinBox on met a jour Cinespace
///////////////////////////////////////////////////////////
void ConfDialog::on_doubleSpinBox_lon_valueChanged(double v)
{
    if (notinitok) return;
    Terre::setLongiLati(toRadians*v, Terre::latobs);
    cine->majCine();
}

void ConfDialog::on_doubleSpinBox_lat_valueChanged(double v)
{
    if (notinitok) return;
    Terre::setLongiLati(Terre::longobs, toRadians*v);
    cine->majCine();
}


void ConfDialog::on_horizontalSliderPrecision_valueChanged(int value)
{
    cine->precision = pow(10., -3. - value/10.);
    ui->labelPrecision->setText(QString::number(cine->precision,'e', 2));
}
