#include "etoile.h"
#include <QString>
#include <QFile>
#include <QTextStream>
#include <QMessageBox>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <iostream> // pour isspsace
#include "reperes.h"
#include "terre.h"
#include "../mathutil/sp3.h"
#include "../utils/angles/angles.h"
#include <vector>
using std::vector;

static double toRadians = (M_PI/180.);

// Enlève les blancs de tête et de queue. Renvoie un pointeur sur le début valide
// et met 0 après le dernier caractère non espace
static char *trimme(char *str)
{
    int first = 0, last = strlen(str);
    while (first < last && isspace(str[first])) first++;
    while (last > first && isspace(str[last-1]))last--;
    str[last]=0;
    return str+first;
}

static char *strGrec[] = {"alp","bet","gam","del","eps","zet","eta","the","iot","kap","lam","mu","nu","xi","omi",
                          "pi","rho","sig","tau","ups","phi","chi","psi","ome"};

// ATTENTION : Les codes UTF16 et UTF8 des lettres grecques sont différents. Les codes UTF16 sur 2 octets commencent tous
// (pour les lettres grecques) par 0x03, alors que les codes UTF8 également sur 2 octets commencent par 0xCE pour les
// majuscules et les minuscules avant "pi" et à partir de "pi" ils commencent par 0xCF et diffèrent aussi au niveau du 2ème
// octet 0x80 au lieu de 0xA0, etc...

static char minGrec[] = {0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6,
                         0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC,
                         0xBD, 0xBE, 0xBF,
                         0x80, 0x81, 0x83, 0x84, 0x85,
                         0x86, 0x87, 0x88, 0x89};

static char majGrec[] = {0x91, 0x92, 0x93, 0x94, 0x95, 0x96,
                         0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C,
                         0x9D, 0x9E, 0x9F,
                         0xA0, 0xA1, 0xA3, 0xA4, 0xA5,
                         0xA6, 0xA7, 0xA8, 0xA9};

const static int nbGrecs = sizeof(minGrec)/sizeof(char);

/*
 * Construit un string unicode UTF8 str en remplaçant dans le string original sorig le nom de la lettre grecque par son symbole
 * Si le flag fmaj est > 0, on renvoie le symbole majuscule
 * s'il est < 0  on renvoie le symbole minuscule,
 * s'il est == 0 on renvoie le symbole minuscule ou majuscule suivant que la première lettre de son nom est
 * majuscule ou minuscule.
 */
static void strToGreek(char *str, char *sorig, int fmaj)
{
    int i, k, n, l = strlen(sorig);
    char smin[4];
    int majus = isupper(sorig[0]);
    // n = nombre de lettres, car il y a peut des chiffres derriere
    for(i = n = 0; i < l; i++) if(isalpha(sorig[i])) n++;
    strncpy(smin, sorig, n); smin[n] = 0; // Les premières lettres transmises
    // Conversion en minuscule
    for(i = 0; i < n; i++) if(isupper(smin[i])) smin[i] = tolower(smin[i]);
    // Recherche du nom dans le tableau
    k = -1;
    for (i = 0; i < nbGrecs; i++) if(strcmp(smin ,strGrec[i]) == 0) {k=i ; break;}
    // Si pas trouvé, retour chaine vide
    if (k < 0) {str[0] = 0; return;}
    // Mise en place utf8 grec
    str[0] = fmaj > 0 || (fmaj == 0 && majus) || k < 15 ? 0xCE : 0xCF;
    if (fmaj < 0) str[1] = minGrec[k];
    else if (fmaj > 0) str[1] = majGrec[k];
    else str[1] = majus ? majGrec[k] : minGrec[k];
    // ajout des caractères qui suivent
    if (l > n) strcpy(str+2, trimme(sorig+n));
    else str[2] = 0;
    return;
}

// Information sur l'étoile
char *infoStar(char *buf, Etoile *e, double t_y)
{
    char str1[80], str2[80];
    int h,d,m;
    double s;
    int n;
    if(e->ok)
    {
        n = sprintf(buf,"Etoile ");
        if (strlen(e->nom)>0) n += sprintf(buf+n,"%s :\n", e->nom);
        else n += sprintf(buf+n,"%s :\n", e->nameEcons);
        n += sprintf(buf+n,"Coordonnées équatoriales à J2000 dans J2000 : %s, %s\n",
                     rad2hms(str1,e->ra0,&h,&m,&s), rad2dms(str2, e->dec0,&d,&m,&s));
        e->corrMouvement(t_y);
        n += sprintf(buf+n,"Coordonnées équatoriales année %lf dans J2000 : %s, %s\n", t_y+2000.,
                     rad2hms(str1,e->ra0t,&h,&m,&s), rad2dms(str2, e->dec0t,&d,&m,&s));

        // Changement coordonnées repère céleste J2000 en repère céleste de la date
        double T = t_y/100.;
        Reperes::majEquatMoy(0., T) ;
        e->compEquatMoyDate();
//        e->coordEquatMoyDate();
        n += sprintf(buf+n,"Coordonnées équatoriales moyennes dans rep.date : %s, %s\n",
                     rad2hms(str1,e->ra,&h,&m,&s), rad2dms(str2, e->dec,&d,&m,&s));
// Coordonnées Equatoriales corrigées de la nutation
        Reperes::obliquite(T);
        Reperes::majEclipVrai(T, true);
//        e->coorEquatVraiDate();
        e->compEquatVraiDate();
        n += sprintf(buf+n,"Coordonnées équatoriales vraies dans rep.date : %s, %s\n",
                     rad2hms(str1,e->ra,&h,&m,&s), rad2dms(str2, e->dec,&d,&m,&s));
// Coordonnées Equatoriales corrigées de l'abberration annuelle
        Terre::posSoleil(t_y*365.25, 1.e-5); // Mise à jour longitude écliptique soleil
//        e->coordEquatApparent(true);
        e->coordEquatApparent(true);
        n += sprintf(buf+n,"Coordonnées équatoriales apparentes dans rep.date : %s, %s\n",
                     rad2hms(str1,e->ra,&h,&m,&s), rad2dms(str2, e->dec,&d,&m,&s));
    }
    else sprintf(buf,"");
    return buf;
}

#define MAXSTARS 9111  // 0 à 9110
#define MAXCONSTELS 54 // pour le moment
static Etoile tabEtoiles[MAXSTARS];
static Constellation tabConstels[MAXCONSTELS];

// lecture du Bright Star Catalogue et decodage des champs pour remplir les structures des objets Etoile.
// Renvoi le nombre d'étoiles renseignées dans le tableau des étoiles, qui doit être égal à 9110.
// La première étoile (indice 0) n'est pas renseignée pour pouvoir garder la numérotation originale
// du Bright Star Catalogue.
// 14 objets sont mal renseignés. Pour ces objet le champ ok vaut false. Pour les autres, il vaut true.
// Les coordonnées initiales des étoiles sont relatives à la date J2000.0
static int LoadCatalogV50()
{
    int nbEtoiles = 0;
    QFile filein(":/catalogV50", NULL);
    if (!filein.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::warning(NULL,"ERREUR","Erreur à l'accès du catalogue d'étoile V50");
        return 0;
    }
    // Lecture du fichier
    const char *line;
    int nRead = 0;
    char *s, str[16], snRead[8], *s1, sflam[4], *s2 ;
    int HH, MM, DD, MD, SD;
    double val, secondes, vx, vy;

    QTextStream in(&filein);
    QString qline;
    QByteArray qba;
    while (true)
    {
        qline = in.readLine();
        if(qline.isNull()) break;
        qba = qline.toLatin1(); line = qba.constData();
        // Traitement avec les fonctions pures C.
        if (strlen(line) < 107) continue; // car ligne incomplete
        // Lecture numero de l'étoile dans le catalogue
        strncpy(snRead, line,4); snRead[4] = 0; s1 = trimme(snRead);
        nRead = atoi(s1);
        if (nRead < 1 || nRead >= MAXSTARS)
        {
            QMessageBox::warning(NULL,"ERREUR","Numéro d'étoile hors limte\n"
                                               "dans le catalogue d'étoiles V50");
            continue;
        }
        Etoile *et = &tabEtoiles[nRead];
        et->lastYcorMouv = 0.; // date des coordonnées par rapport à J2000.0
        et->bsNumber = nRead;
        // Lecture nom (bsName) dans le catalogue
        strncpy(et->bsName, line+4, 10); et->bsName[10] = 0;
        // Extraction flamsteed de bsName
        strncpy(sflam, et->bsName, 3); sflam[3] = 0; s2 = trimme(sflam);
        et->flamsteed = (strlen(s2) > 0) ? atoi(s2) : 0;
        // Extraction bayer de bsName
        strncpy(str, et->bsName+3, 4); str[4] = 0; s = trimme(str);
        if(strlen(s) > 0) strcpy(et->bayer,s);
        // Extraction constel de bsName
        strncpy(str, et->bsName+7, 3); str[3] = 0; s = trimme(str);
        if (strlen(s) > 0) strcpy(et->constel,s);
        // Fabrication du short Greek Name à ma sauce constitué par :
        // - le caractère grec unicode minuscule si la dénomination grecque (bayer) est présente
        // - sinon le numero flamsteed si > 0
        // - sinon ? suivi du bsNumber

        if(et->bayer != NULL && strlen(et->bayer) > 0) strToGreek(et->sgName, et->bayer, -1);
        else if (et->flamsteed != 0 ) strcpy(et->sgName, s2);
        else {strcpy(et->sgName, "?"); strcat(et->sgName, s1);}
        // consName
        strcpy(et->nameEcons, et->sgName); strcat(et->nameEcons, ":"); strcat(et->nameEcons, et->constel);
        //Ascension droite
        strncpy(str,line+75,2); str[2] = 0; HH = atoi(str);
        strncpy(str,line+77,2); str[2] = 0; MM = atoi(str);
        strncpy(str,line+79,4); str[4] = 0; secondes = strtod(str, NULL);
        et->ra = et->ra0t = et->ra0 = toRadians*(15.*(HH + MM/60. + secondes/3600.));
        //Declinaison
        strncpy(str,line+84,2); str[2] = 0; DD = atoi(str);
        strncpy(str,line+86,2); str[2] = 0; MD = atoi(str);
        strncpy(str,line+88,2); str[2] = 0; SD = atoi(str);
        et->dec0 = toRadians*(DD + MD/60. + SD/3600.);
        if (line[83] == '-') et->dec0 = -et->dec0;
        et->dec = et->dec0t = et->dec0;
        double cosDec = cos(et->dec);
        // Mouvement propre en ascension droite et declinaison dans J2000 en radians / an
        strncpy(str,line+148,6); str[6] = 0; sscanf(str, "%lf",&val); vx = toRadians*val/3600.;
        strncpy(str,line+154,6); str[6] = 0; sscanf(str, "%lf",&val); vy = toRadians*val/3600.;
        et->vMod = sqrt(vx*vx + vy*vy);
        et->vRA = vx/cosDec;
        et->vDE = vy;
        // Calcul vecteur directeur unitaire dans Rc
        et->uCS[0] = et->uCSo[0] = cos(et->ra)*cosDec;
        et->uCS[1] = et->uCSo[1] = sin(et->ra)*cosDec;
        et->uCS[2] = et->uCSo[2] = sin(et->dec);
        //Magnitude
        strncpy(str,line+102,5); str[5] = 0; et->mag = strtod(str, NULL);
        // Signalement étoiles mal renseignées (on ne teste que l'ascension droite)
        if(line[76]==' ' || line[78]==' ' || line[80]==' ') et->ok = false;
        else et->ok = true;
        nbEtoiles++;
    } // Fin du while
    filein.close();
    return nbEtoiles;
}

/*
 * Lecture d'un ensemble de nom d'étoiles ayant un nom en français (presque toutes)
 * Recherche de ces étoiles dans le tableau des étoiles pour remlir le champ de leur
 * nom français.
 */
static int LectureNomsStars()
{
    if (Etoile::nbGets == 0)
    {
        QMessageBox::warning(NULL,"ERREUR","La classe étoile n'est pas encore initialisée !");
        return 0;
    }
    QFile filein(":/100Stars", NULL);
    if (!filein.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::warning(NULL,"ERREUR","Erreur à l'accès du fichier contenant les noms de 100 étoiles");
        return 0;
    }
    const char *line;
    char str[8], name[16], *sn1; name[15] = 0;
    Etoile *e;
    int numStar, nbread = 0;
    QTextStream in(&filein);
    QString qline;
    QByteArray qba;
    while (true)
    {
        qline = in.readLine();
        if(qline.isNull()) break;
        qba = qline.toLatin1(); line = qba.constData();
        // Traitement avec les fonctions pures C.
        if (line[0] == '#' || line[0] == '-') continue; // Saute commentaire
        // Lecture numero dans le Bright Star Catalogue
        strncpy(str,line+73,4); str[4] = 0; numStar = atoi(str);
        if (numStar < 1 || numStar >= MAXSTARS) continue;
        // Lecture du nom
        strncpy(name, line+78, 15);
        if ((sn1 = strchr(name, '\r')) != NULL) *sn1 = 0; // On termine la ligne au \r ou \n
        if ((sn1 = strchr(name, '\n')) != NULL) *sn1 = 0;
        sn1 = trimme(name);
        if (strlen(sn1) == 0) continue;
        e = &tabEtoiles[numStar];
        strcpy(e->nom, sn1);
        nbread++;
    } // Fin while
    filein.close();
    return nbread;
}

// Initialisation des statics
int Etoile::nbGets = 0;
double Etoile::lastYcorAberr = -1.e+20; // Aberration non corrigée à l'origine
Etoile *Etoile::init()
{
    // Lecture catalogue étoiles V50
    int nbEtoiles = LoadCatalogV50();
    if (nbEtoiles == 0)
    {
        QMessageBox::warning(NULL,"ERREUR","Echec lecture Catalogue étoile V50");
        return NULL;
    }
    Etoile::nbGets = nbEtoiles;
    // Lecture nom français des principales étoiles
    int nbNoms = LectureNomsStars();
    if (nbNoms == 0) QMessageBox::warning(NULL,"ERREUR","Echec lecture fichier nom des étoiles");
    return &tabEtoiles[0];
}

Etoile::Etoile()
{
    ok = false;
    bsNumber = 0;
    bsName[0] = 0;	// Nom lu dans le Bright Star Catalogue
    bayer[0] = 0;	// Lettre étoile dans la constellation selon Bayer si renseigné
    constel[0] = 0;	// Nom abrégé de la constellation si renseigné
    nom[0] = 0;         // Nom commun de l'étoile si renseigné
    sgName[0] = 0;  // shortGreekName à ma sauce
}

// Correction mouvement propre de l'étoile à la date yJ (année julienne) après J2000
void Etoile::corrMouvement(double yJ)
{
    ra0t = ra0 + vRA*yJ;
    dec0t = dec0 + vDE*yJ;
    double cosDec = cos(dec0t);
    uCSo[0] = cos(ra0t)*cosDec;
    uCSo[1] = sin(ra0t)*cosDec;
    uCSo[2] = sin(dec0t);
    lastYcorMouv = yJ;
}

// Calcul composantes et coordonnées équatoriales moyennes repère céleste de la date
void Etoile::compEquatMoyDate()
{
// Ce calcul suppose que Reperes::majEquatMoy(0., sJ) a été appelé au préalable
    m3v3(uCS, Reperes::m3_CC21, uCSo);
    dec = asin(uCS[2]);
    ra = atan2(uCS[1], uCS[0]);
    return;
}

// Calcul coordonnées et composantes équatoriales moyennes repère céleste de la date
void Etoile::coordEquatMoyDate()
{
// Ce calcul suppose que Reperes::majEquatMoy(0., sJ) a été appelé au préalable

    double cosDec, sinDec;
    Reperes::chgCoordsEquat(&ra, &dec, &sinDec, ra0t, dec0t, true);
    cosDec = cos(dec);
    uCS[0] = cos(ra)*cosDec;
    uCS[1] = sin(ra)*cosDec;
    uCS[2] = sinDec;
    return;
}

// Calcul composantes et coordonnées équatoriales vraies repère céleste de la date
void Etoile::compEquatVraiDate()
{
// Ce calcul suppose que Reperes::majEclipVrai(sJ, v/f) a été appelé au préalable
    m3v3(uCS, Reperes::m3_CCvm, uCS);
    ra = atan2(uCS[1], uCS[0]);
    dec = asin(uCS[2]);
    return;
}

// Calcul coordonnées et composantes équatoriales vraies repère céleste de la date
void Etoile::coordEquatVraiDate()
{
// Ce calcul suppose que Reperes::majEclipVrai(sJ, v/f) a été appelé au préalable
    double deltaAlf, deltaDec;
    Reperes::deltaCoordEquatNutation(&deltaAlf, &deltaDec, ra, dec);
    ra += deltaAlf;
    dec += deltaDec;
    double cosDec = cos(dec);
    uCS[0] = cos(ra)*cosDec;
    uCS[1] = sin(ra)*cosDec;
    uCS[2] = sin(dec);
    return;
}

#define Kappa 0.99364e-4  // 20.4952 secondes d'arc converti en radians

// Calcul coordonnées et composantes équatoriales apparentes repère céleste de la date
void Etoile::coordEquatApparent(bool precismax)
{
// Ce calcul suppose que Terre::solLongiEclip est à jour, ainsi que l'obliquité
// l'influence de l'ellipticité de l'orbite de la Terre est négligée
    double deltaAlf, deltaDec;
    double cosa = cos(ra), sina = sin(ra), cosDec = cos(dec), sinDec = sin(dec);
    double A, C, D;
    A = cosa*Reperes::coseps;
    C = Reperes::sineps*cosDec - Reperes::coseps*sina*sinDec;
    D = cosa*sinDec;
    deltaAlf = -Kappa*(A*Terre::cosLE + sina*Terre::sinLE)/cos(dec);
    deltaDec = -Kappa*(C*Terre::cosLE + D*Terre::sinLE);
    if(precismax)
    {
        double p = Terre::solLongiPeri + M_PI, cosp = cos(p), sinp = sin(p);
        double k = Kappa*Terre::e;
        deltaAlf += k*(A*cosp + sina*sinp)/cos(dec);
        deltaDec += k*(C*cosp + D*sinp);
    }
    ra += deltaAlf;
    dec += deltaDec;
    cosDec = cos(dec);
    uCS[0] = cos(ra)*cosDec;
    uCS[1] = sin(ra)*cosDec;
    uCS[2] = sin(dec);
    return;
}

// Calcul composantes et coordonnées équatoriales apparentes repère céleste de la date
void Etoile::compEquatApparent(bool precismax)
{
// Ce calcul suppose que Terre::solLongiEclip est à jour, ainsi que l'obliquité
// l'influence de l'ellipticité de l'orbite de la Terre est négligée
double sinBeta, beta, lambda, deltaL, sinLamd, cosBeta, deltaPsi, deltaBeta;
// Calcul coordonnées écliptiques lambda, beta
    sinBeta = -Reperes::sineps*uCS[1] + Reperes::coseps*uCS[2];
    beta = asin(sinBeta);
    lambda = atan2(Reperes::coseps*uCS[1] + Reperes::sineps*uCS[2],  uCS[0]);
// Calcul variation coordonnées écliptiques deltaPsi, deltaBeta
    deltaL = Terre::solLongiEclip - lambda;
    deltaPsi = -Kappa*cos(deltaL)/cos(beta);
    deltaBeta = -Kappa*sinBeta*sin(deltaL);
    if(precismax)
    {
        deltaL = Terre::solLongiPeri + M_PI - lambda;
        double k = Kappa*Terre::e;
        deltaPsi += k*cos(deltaL)/cos(beta);
        deltaBeta += k*sinBeta*sin(deltaL);
    }
// Nouvelle coordonnées écliptiques
    lambda += deltaPsi;
    beta += deltaBeta;
// Nouvelles composantes et coordonnées équatoriales
    cosBeta = cos(beta);
    sinBeta = sin(beta);
    sinLamd = sin(lambda);
    uCS[0] = cos(lambda)*cosBeta;
    uCS[1] = Reperes::coseps*sinLamd*cosBeta-Reperes::sineps*sinBeta;
    uCS[2] = Reperes::sineps*sinLamd*cosBeta + Reperes::coseps*sinBeta;
    ra = atan2(uCS[1], uCS[0]);
    dec = asin(uCS[2]);
    return;
}

// Misa à jour coordonnées pour la date te_j donnée en jours juliens par rapport à J2000.
void Etoile::misajour(double te_j, double precision, bool corrNut)
{
    double te_y = te_j/365.25;
    double T = te_y/100.;
    Etoile *e;
    // Correction des coordonnées stellaires en fonction de leur mouvement propre et
    // de la precession des équinoxes
    if (fabs(T - Reperes::lastsJc2)*Reperes::varPrecSiecle >= precision)
    {
        // Calcul paramètres correction Repère J2000 -> repère de la date
        Reperes::majEquatMoy(0., T);
        // Application correction à toutes les étoiles
        for (int i = 1; i <= Etoile::nbStars; i++)
        {
            e = &tabEtoiles[i];
            if (!e->ok) continue;
            // Correction déplacement annuel mouvement propre dans J2000.
            if (fabs(te_y - e->lastYcorMouv)*e->vMod >= precision)
                e->corrMouvement(te_y);
            // Calcult coordonnées équatoriales dans repère céleste de la date
            e->compEquatMoyDate();
        }
    }
    // Correction de la nutation
    if(corrNut && precision < Reperes::errBradleyMax &&
            fabs(T - Reperes::lastCorNut)*Reperes::errBradSiecle >= precision)
    {
        // Calcul paramètres correctifs
        Reperes::majEclipVrai(T, false);
        // Application à toutes les étoiles
        for (int i = 1; i <= Etoile::nbStars; i++)
        {
            e = &tabEtoiles[i];
            if (!e->ok) continue;
            e->compEquatVraiDate();
        }
    }
    // Correction de l'aberration annuelle
    if (precision < Kappa && fabs(te_y - lastYcorAberr)*Kappa >= precision)
    {
        Etoile *e;
        Terre::posSoleil(te_j, precision);
        for (int i = 1; i <= Etoile::nbStars; i++)
        {
            e = &tabEtoiles[i];
            if (!e->ok) continue;
            e->compEquatApparent(precision < 0.5e-6);
        }
        lastYcorAberr = te_y;
    }
}

//-------------------------------------------------
//-------------      CONSTELLATIONS   -------------
//-------------------------------------------------
// Initialisation du static
int Constellation::nbConstels = 0;

// Routine statique qui génère le tableau statique des constellations
Constellation *Constellation::init()
{
    if (Etoile::nbGets == 0)
    {
        QMessageBox::warning(NULL,"ERREUR","La classe étoile n'est pas encore initialisée !");
        return NULL;
    }
    QFile filein(":/constels", NULL);
    if (!filein.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QMessageBox::warning(NULL,"ERREUR","Erreur à l'accès du fichier décrivant les constellations");
        return NULL;
    }
    // Pour le calcul du barycentre
    double xs, ys, zs, som, cosDec;
    Etoile *e;
    // Lecture du fichier
    int nbConstel = 0;
    Constellation *curCons;
    vector <int> starIndexes;
    vector <int> oris;
    vector <int> dess;
    int idx, i1,i2;
    bool ok = true;
    char *st;

    // La présence éventuelle d'UTF-8 dans le nom augmente le nombre d'octets/au nb de colonnes du fichier
    const char *line;
    char curLine[256];
    QTextStream in(&filein);
    QString qline;
    QByteArray qba;
    while (true)
    {
        qline = in.readLine();
        if(qline.isNull()) break;
        qba = qline.toLatin1(); line = qba.constData(); strcpy(curLine, line);
        // si la ligne commence par "constellation", on crée une nouvelle constellation
        if(strncmp(curLine,"Constellation",13) != 0) {ok = false; break;}
        if (nbConstel >= MAXCONSTELS) break;
        curCons = &tabConstels[nbConstel];
        // lecture Nom
        strncpy(curCons->shortName, curLine+14, 3); curCons->shortName[3] = 0;
        strcpy(curCons->frenchName, trimme(curLine+20));

        // Lecture ligne suivante : les numéros des étoiles
        qline = in.readLine();
        if(qline.isNull()) {ok = false; break;}
        qba = qline.toLatin1(); line = qba.constData(); strcpy(curLine, line);
        st = strtok(trimme(curLine)," \t");
        while (st != NULL)
        {
            idx = atoi(trimme(st));
            if (idx <= 0 || idx > Etoile::nbStars)
            { fprintf(stderr, "Erreur : numero étoile, hors catalogue V50"); ok = false; break;}
            starIndexes.push_back(idx);
            st = strtok(NULL," \t");
        }
        if (!ok) break;

        // Lecture ligne suivante : la liste des segments
        qline = in.readLine();
        if(qline.isNull()) {ok = false; break;}
        qba = qline.toLatin1(); line = qba.constData(); strcpy(curLine, line);

        st = strtok(trimme(curLine)," \t");
        while (st != NULL)
        {
            i1 = atoi(trimme(st));
            if ((st = strtok(NULL," \t")) == NULL) {ok = false; break;}
            i2 = atoi(trimme(st));
            oris.push_back(i1);
            dess.push_back(i2);
            st = strtok(NULL," \t");
        }
        if (!ok) break;

        // Copie liste étoiles dans le tableau des index
        curCons->starsIndexSize = starIndexes.size();
        curCons->starsIndex = new int[curCons->starsIndexSize];
        for(int i=0; i < curCons->starsIndexSize; i++) curCons->starsIndex[i] = starIndexes[i];
        starIndexes.clear();
        // Copie liste segments dans le tableau des liaisons
        curCons->linesSize = oris.size();
        curCons->linesOri = new int[curCons->linesSize];
        for(int i=0; i < curCons->linesSize; i++) curCons->linesOri[i] = oris[i];
        oris.clear();
        curCons->linesDes = new int[curCons->linesSize];
        for(int i=0; i < curCons->linesSize; i++) curCons->linesDes[i] = dess[i];
        dess.clear();
        // Calcul barycentre de la constellation
        xs = ys = zs = som = 0.;
        for (int i=0; i < curCons->starsIndexSize; i++)
        {
            e = &tabEtoiles[curCons->starsIndex[i]];
            // Calcul barycentre constellation
            xs += e->uCS[0];
            ys += e->uCS[1];
            zs += e->uCS[2];
        }
        som = sqrt(xs*xs+ys*ys+zs*zs);
        curCons->decLabel = asin(zs/som);
        curCons->raLabel = atan2(ys, xs);
        // Calcul vecteur directeur unitaire dans Rc
        cosDec = cos(curCons->decLabel);
        curCons->uCS[0] = cos(curCons->raLabel)*cosDec;
        curCons->uCS[1] = sin(curCons->raLabel)*cosDec;
        curCons->uCS[2] = sin(curCons->decLabel);
        // Cette constellation est fini, on l'archive
        nbConstel++;
    } // fin du while
    filein.close();
    Constellation::nbConstels = nbConstel;
    return  &tabConstels[0];
}

/*
 *  Nom de la constellation suivi de la liste de ses étoiles
*/
char *Constellation::toNewString()
{
    Etoile *e;
    int nf = strlen(frenchName);
    int n = nf + 2 + 15*starsIndexSize;
    char *str = new char[n];
    strcpy(str, frenchName); strcat(str, " :");
    for (int i=0; i < starsIndexSize; i++)
    {
        e = &tabEtoiles[starsIndex[i]];
        strcat(str, " ");
        // Si le nom de la constelation de l'étoile est différent, on l'ajoute
        if (strlen(e->constel) > 0 && strcmp(e->constel, shortName)) strcat(str, e->nameEcons);
        else strcat(str,e->sgName); // sinon on n'ajoute que son short greek name.
        // Ajout du grand nom de l'étoile si existe
        if(strlen(e->nom) > 0)
        {
            strcat(str,"(");
            strcat(str,e->nom);
            strcat(str,")");
        }
    }
    return str;
}

