/*-------------  MODE D'EMPLOI  ----------------------------------------------------
   Pour créer la displayList du dessin OPenGL d'un satellite de taille 1. il suffit
   d'appeler la routine :
        GLuint DL_sat = glNewPleiade(1.);
   Pour la dessiner il suffit de faire :
        glCallList(DL_sat);
   Pour la détruire il suffit de faire :
        glDeleteLists(DL_sat,1);
---------------------------------------------------------------------------------------*/
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#ifndef M_PI
#define M_PI   3.14159265358979323846
#endif
#define RAD2DEG (180./M_PI)
#define DEG2RAD (M_PI/180.)


static void setv3f(GLfloat *v, GLfloat x, GLfloat y, GLfloat z) {v[0]=x; v[1]=y; v[2]=z;}

static void setv3fv(GLfloat *v, GLfloat *w) {v[0]=w[0];v[1]=w[1];v[2]=w[2];}

static void plusv3f(GLfloat *v, GLfloat *w) {v[0]+=w[0];v[1]+=w[1];v[2]+=w[2];}

static void triPt2Norm(GLfloat *n, GLfloat w[3][3])
{
    double a[3], b[3], x, y, z, s;
    a[0] = w[0][0] - w[1][0];
    a[0] = w[0][1] - w[1][1];
    a[0] = w[0][2] - w[1][2];
    b[0] = w[1][0] - w[2][0];
    b[0] = w[1][1] - w[2][1];
    b[0] = w[1][2] - w[2][2];
    x = a[1]*b[2] - a[2]*b[1];
    y = a[2]*b[0] - a[0]*b[2];
    z = a[0]*b[1] - a[1]*b[0];
    s = sqrt(x*x+y*y+z*z);
    if(s == 0.) {n[0]=1.; n[1] = n[2] = 0.; return;}
    n[0] = x/s; n[1] = y/s; n[2] = z/s;
    return;
}

GLuint glNewPleiade(GLfloat size)
{
    int i,j,k;
    double pis3 = M_PI/3.;
    double pis6 = M_PI/6.;

    GLfloat cpis3 = (GLfloat) cos(pis3);
    GLfloat spis3 = (GLfloat) sin(pis3);
    GLfloat cpis6 = (GLfloat) cos(pis6);
    GLfloat spis6 = (GLfloat) sin(pis6);

    GLfloat r0 = 0.3f*size; 	// rayon base externe module propulsion
    GLfloat r1 = 0.4f*size; 	// rayon base interne module propulsion
    GLfloat h0 = 0.3f*size; 	// hauteur module propulsion
    GLfloat h1 = 1.6f*size; 	// hauteur corps hexagonal
    GLfloat r2 = 0.9f*size; 	// rayon corps hehagonal
    GLfloat h2 = 0.85f*size;	// hauteur support instrument
    GLfloat r3 = 0.625f*size;	// rayon bas support instrument
    GLfloat dec = 0.05f*size; // dï¿½calage
    GLfloat r4 = 0.75f*size;
    GLfloat h4 = 0.3f*size;	// hauteur baffle
    GLfloat r5 = 0.325f*size;	// rayon baffle
    GLfloat lp = 2.1f*size;	// longueur panneau
    GLfloat ranten = 0.05f*size;

    GLfloat r0c = r0*cpis3;
    GLfloat r0s = r0*spis3;
    GLfloat r1c = r1*cpis3;
    GLfloat r1s = r1*spis3;
    GLfloat r2c = r2*cpis3;
    GLfloat r2s = r2*spis3;
    GLfloat r3c = r3*cpis3;
    GLfloat r3s = r3*spis3;
    GLfloat r4c = r4*cpis3;
    GLfloat r4s = r4*spis3;
    GLfloat r5c = r5*cpis3;
    GLfloat r5s = r5*spis3;

    GLfloat lpc = lp*cpis6;
    GLfloat lps = lp*spis6;    

    GLfloat z0 = -(h1 + h0);	// base externe module propulsion
    GLfloat z1 = -h1;		// base basse corps hexagonal
    GLfloat z2 = 0;		// separation corps/support instrument
    GLfloat z3 = h2;		// basse support instrument
    GLfloat z3d = z3 + dec;     // fond baffle
    GLfloat z4 = h2 + h4;       // extremite baffle

    GLfloat demifauch_hotspot = 5.*DEG2RAD;
    GLfloat demi_angle_avant_hotspot = 30.*DEG2RAD;
//    GLfloat demifauch_viseur = 25.*DEG2RAD;
    GLfloat demifauch_viseur = 2.5*DEG2RAD;
    GLfloat ray = 1000000.f*size; // 1000 km
    GLfloat rtan_viseur = (GLfloat) (ray*tan(demifauch_viseur));
    GLfloat ravant =  (GLfloat) (ray*tan(demi_angle_avant_hotspot));
    GLfloat rtan_hotspot = (GLfloat) (ravant*tan(demifauch_hotspot));

    GLfloat color_cover[] = {1.f, 0.78f, 0.2f} ;
    GLfloat color_anten[] = {0.8f, 0.65f, 0.15f} ;
    GLfloat color_pansol[] = {0.106f, 0.137f, 0.318f};
    GLfloat color_tube[] = {0.765f ,0.765f, 0.765f};
    GLfloat color_baff[] = {0.400f ,0.400f, 0.400f};
    GLfloat color_bluk[] = {0.05f, 0.05f, 0.15f};
    GLfloat color_red[] = {1.0f, 0.0f, 0.0f, 1.f};
    GLfloat eteinte[] = {0.0f, 0.0f, 0.0f, 1.0f};

    // Tableau des points decrivant le satellite et quelques normales
    GLfloat vect_hexa[6][3] = {
        {cpis6, spis6, 0.f},
        {0.f, 1.f, 0.f},
        {-cpis6, spis6, 0.f},
        {-cpis6, -spis6, 0.f},
        {0.f, -1.f, 0.f},
        {cpis6, -spis6, 0.f}
    };
    GLfloat vect_Zneg[3] = {0.f, 0.f, -1.f};
    GLfloat vect_Zpos[3] = {0.f, 0.f, 1.f};
    GLfloat vect_Xpos[3] = {1.f, 0.f, 0.f};

    int is0 = 0, is1 = 1*6, is2 = 2*6, is3 = 3*6, is4 = 4*6, is5 = 5*6;
    GLfloat tabSom[36][3] = {
        // base externe module de propulsion
        { r0 , 0.f,  z0},
        { r0c, r0s,  z0},
        {-r0c, r0s,  z0},
        {-r0 , 0.f,  z0},
        {-r0c,-r0s,  z0},
        { r0c,-r0s,  z0},
        // base interne module de propulsion
        { r1 , 0.f,  z1},
        { r1c, r1s,  z1},
        {-r1c, r1s,  z1},
        {-r1 , 0.f,  z1},
        {-r1c,-r1s,  z1},
        { r1c,-r1s,  z1},
        // base basse corps hexagonal
        { r2 , 0.f,  z1},
        { r2c, r2s,  z1},
        {-r2c, r2s,  z1},
        {-r2 , 0.f,  z1},
        {-r2c,-r2s,  z1},
        { r2c,-r2s,  z1},
        // separation corps/support instrument
        { r2 , 0.f,  z2},
        { r2c, r2s,  z2},
        {-r2c, r2s,  z2},
        {-r2 , 0.f,  z2},
        {-r2c,-r2s,  z2},
        { r2c,-r2s,  z2},
        // base support instrument
        { r3 , 0.f,  z3},
        { r3c, r3s,  z3},
        {-r3c, r3s,  z3},
        {-r3 , 0.f,  z3},
        {-r3c,-r3s,  z3},
        { r3c,-r3s,  z3},
        // Les 3 panneaux solaires
        { lpc, lps,0.f},
        { lpc, lps,0.f},
        {-lpc, lps,0.f},
        {-lpc, lps,0.f},
        { 0.f,-lp ,0.f},
        { 0.f,-lp ,0.f},
    };

    plusv3f(tabSom[is5+0], tabSom[is2+0]);
    plusv3f(tabSom[is5+1], tabSom[is2+1]);
    plusv3f(tabSom[is5+2], tabSom[is2+2]);
    plusv3f(tabSom[is5+3], tabSom[is2+3]);
    plusv3f(tabSom[is5+4], tabSom[is2+4]);
    plusv3f(tabSom[is5+5], tabSom[is2+5]);

    // Base externe module de propulsion et base basse corps hexagonal
    GLfloat *tabFan[16];
    GLfloat *normFan[16];
    GLfloat bid0[] = { 0.f , 0.f,  z0};
    tabFan[0] = bid0;
    normFan[0] = vect_Zneg;
    for (i = 0; i < 6; i++)
    {
        tabFan[1+i] = tabSom[is0+5-i];
        normFan[1+i] = vect_Zneg;
    }
    tabFan[7] = tabFan[1];
    normFan[7] = vect_Zneg;

    // base basse corps hexagonal
    GLfloat bid1[] = { 0.f , 0.f,  z1};
    tabFan[8] = bid1;
    normFan[8] = vect_Zneg;
    for (i = 0; i < 6; i++)
    {
        tabFan[9+i] = tabSom[is2+5-i];
        normFan[9+i] = vect_Zneg;
    }
    tabFan[15] = tabFan[9];
    normFan[15] = vect_Zneg;

    // Parois module propulsion, corps hexa externe et support instrument -> qa
    GLfloat *normQua[72];
    GLfloat *tabQua[72];
    k = 0;
    // parois module propulsion
    for (i = 0; i < 6; i++)
    {
        j = i == 5 ? 0 : i+1;
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is1+i];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is0+i];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is0+j];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is1+j];
    }
    // parois corps externe
    for (i = 0; i < 6; i++)
    {
        j = i == 5 ? 0 : i+1;
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is3+i];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is2+i];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is2+j];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is3+j];
    }
    // parois support instrument
    for (i = 0; i < 6; i++)
    {
        j = i == 5 ? 0 : i+1;
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is4+i];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is3+i];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is3+j];
        normQua[k] = vect_hexa[i];
        tabQua[k++] = tabSom[is4+j];
    }
    // Les panneaux solaires
    GLfloat *tabQua2[12];
    k = 0;
    for (i = 0; i < 6; i += 2)
    {
        j = i+1;
        tabQua2[k++] = tabSom[is2+i];
        tabQua2[k++] = tabSom[is5+i];
        tabQua2[k++] = tabSom[is5+j];
        tabQua2[k++] = tabSom[is2+j];
    }
    // La plaque triangulaire -> qa3
    GLfloat plaqtri[12][3] = {
        { r5, 0.f,   z3},
        { r4, 0.f,   z3},
        { r5c, r5s,  z3},
        { r3c, r3s,  z3},
        { -r5c, r5s, z3},
        { -r4c, r4s, z3},
        { -r5, 0.f,  z3},
        { -r3, 0.f,  z3},
        { -r5c, -r5s,z3},
        { -r4c, -r4s,z3},
        { r5c, -r5s, z3},
        { r3c, -r3s, z3}

    };
    GLfloat *tabqa3[24];
    tabqa3[23] = tabqa3[0] = plaqtri[0];
    tabqa3[22] = tabqa3[1] = plaqtri[1];
    tabqa3[3] = tabqa3[4] = plaqtri[2];
    tabqa3[2] = tabqa3[5] = plaqtri[3];
    tabqa3[7] = tabqa3[8] = plaqtri[4];
    tabqa3[6] = tabqa3[9] = plaqtri[5];
    tabqa3[11] = tabqa3[12] = plaqtri[6];
    tabqa3[10] = tabqa3[13] = plaqtri[7];
    tabqa3[15] = tabqa3[16] = plaqtri[8];
    tabqa3[14] = tabqa3[17] = plaqtri[9];
    tabqa3[19] = tabqa3[20] = plaqtri[10];
    tabqa3[18] = tabqa3[21] = plaqtri[11];
    // Baffle -> baf, Fond du baffle
    int Nbaf = 12;
    GLfloat normbaf[4*Nbaf][3];
    GLfloat *normtfb[Nbaf+2];
    GLfloat tabbaf[4*Nbaf][3];
    GLfloat tabtfb[Nbaf+2][3];
    setv3f(tabtfb[0],  0.f , 0.f,  z3d);
    normtfb[0] =  vect_Zpos;
    setv3f(tabtfb[1], r5 , 0.f,  z3d); setv3fv(tabtfb[Nbaf+1], tabtfb[1]);
    normtfb[1] =  vect_Zpos;
    normtfb[Nbaf+1] =  vect_Zpos;
    // paroi baffle et fond
    double ang = 2.*M_PI/Nbaf, angi;
    GLfloat cc, ss;
    /* // Version commentee : pour normales orthogonales aux facettes
    double angs2 = ang/2., angips2;
    cc = (GLfloat)cos(angs2);
    ss = (GLfloat)sin(angs2);
     */
    setv3f(tabbaf[0],  r5, 0.f,  z3); setv3fv(tabbaf[4*Nbaf-1], tabbaf[0]);
    setv3f(tabbaf[1], r5, 0.f,  z4); setv3fv(tabbaf[4*Nbaf-2], tabbaf[1]);
    //normbaf[0] = normbaf[1] = new Vector3f(cc, ss, 0.f);
    //normbaf[4*Nbaf-1] = normbaf[4*Nbaf-2] = new Vector3f(cc, -ss, 0.f);
    setv3fv(normbaf[0], vect_Xpos);
    setv3fv(normbaf[1], vect_Xpos);
    setv3fv(normbaf[4*Nbaf-1], vect_Xpos);
    setv3fv(normbaf[4*Nbaf-2],vect_Xpos);
    k = 2;
    for (i = 1; i < Nbaf; i++)
    {
        angi = i*ang;
        cc = (GLfloat)cos(angi);
        ss = (GLfloat)sin(angi);
        r5c = r5*cc;
        r5s = r5*ss;
        /*
        angips2 = angi + angs2;
        cc = (GLfloat)cos(angips2);
        ss = (GLfloat)sin(angips2);
         */
        setv3f(tabbaf[k+1],  r5c, r5s,  z3); setv3f(tabbaf[k+2], r5c, r5s,  z3);
        setv3f(tabbaf[k],  r5c, r5s,  z4); setv3f(tabbaf[k+3], r5c, r5s,  z4);
        //normbaf[k] = normbaf[k+1] = normbaf[k-1];
        //normbaf[k+2] = normbaf[k+3] = new Vector3f(cc, ss, 0.f);
        setv3f(normbaf[k], cc, ss, 0.f);
        setv3f(normbaf[k+1], cc, ss, 0.f);
        setv3f(normbaf[k+2], cc, ss, 0.f);
        setv3f(normbaf[k+3],cc, ss, 0.f);
        k += 4;
        setv3f(tabtfb[i+1], r5c, r5s,  z3d);
        normtfb[i+1] = vect_Zpos;
    }
    // Micro-antenne
    int Npoly = 4;
    GLfloat norm_anten[4*Npoly][3];
    GLfloat pt_anten[4*Npoly][3];
    ang = 2.*M_PI/Npoly;
    double angs2 = ang/2. /* , angips2 */;
    cc = (GLfloat)cos(angs2);
    ss = (GLfloat)sin(angs2);
    setv3f(pt_anten[0], r3+ranten, 0.f,  z3); setv3fv(pt_anten[4*Npoly-1], pt_anten[0]);
    setv3f(pt_anten[1], r3+ranten, 0.f,  z4); setv3fv(pt_anten[4*Npoly-2], pt_anten[1]);
    //norm_anten[0] = norm_anten[1] = new Vector3f(cc, ss, 0.f);
    //norm_anten[4*Npoly-1] = norm_anten[4*Npoly-2] = new Vector3f(cc, -ss, 0.f);
    setv3fv(norm_anten[0], vect_Xpos);
    setv3fv(norm_anten[1], vect_Xpos);
    setv3fv(norm_anten[4*Npoly-1], vect_Xpos);
    setv3fv(norm_anten[4*Npoly-2], vect_Xpos);
    k = 2;
    for (i = 1; i < Npoly; i++)
    {
        angi = i*ang;
        cc = (GLfloat)cos(angi);
        ss = (GLfloat)sin(angi);
        r5c = ranten*cc;
        r5s = ranten*ss;
        /*
        angips2 = angi + angs2;
        cc = (GLfloat)cos(angips2);
        ss = (GLfloat)sin(angips2);
         */
        setv3f(pt_anten[k+1], r3+r5c, r5s,  z3); setv3fv(pt_anten[k+2], pt_anten[k+1]);
        setv3f(pt_anten[k], r3+r5c, r5s,  z4);  setv3fv(pt_anten[k+3], pt_anten[k]);
        //norm_anten[k] = norm_anten[k+1] = norm_anten[k-1];
        //norm_anten[k+2] = norm_anten[k+3] = new Vector3f(cc, ss, 0.f);
        setv3f(norm_anten[k], cc, ss, 0.f);
        setv3fv(norm_anten[k+1], norm_anten[k]);
        setv3fv(norm_anten[k+2], norm_anten[k]);
        setv3fv(norm_anten[k+3], norm_anten[k]);
        k += 4;
    }
    // Viewer : rayon en avant vers la Terre
    GLfloat pt_ray[3][3] = {
        {0.f, 0.f, z4},
        {0.f, -rtan_viseur,ray},
        {0.f, rtan_viseur,ray},
    };
    GLfloat norm_ray[3];
    triPt2Norm(norm_ray, pt_ray);
    // Le hot-sppotter
    GLfloat pt_hsp[3][3] = {
        {0.f, 0.f, z4},
        {ravant, -rtan_hotspot, ray},
        {ravant, rtan_hotspot, ray}
    };
    GLfloat norm_hsp[3];
    triPt2Norm(norm_hsp, pt_hsp);

    GLuint shape = glGenLists(1);
    glNewList(shape, GL_COMPILE);
    {
        glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE); // On ne traite que les faces externes
        glColorMaterial(GL_FRONT, GL_DIFFUSE);      // Mode de reflection
        glCullFace(GL_BACK);                        // Eliminer la face arriere
        glEnable(GL_CULL_FACE);                     // Autorise l'élimination
        glPolygonMode(GL_FRONT, GL_FILL) ;          // Peint qu'une surface
        // panneaux solaires coté externe (idem interne, à l'envers)
        glFrontFace(GL_CW);                         // Avant defini en clock-wise

        glColor3fv(color_pansol);
        glBegin(GL_QUADS);
        for(i=0;i<12;i++)
        {
            glNormal3fv(vect_Zneg);
            glVertex3fv(tabQua2[i]);
        }
        glEnd();
        // panneaux solaires coté satellite
        glFrontFace(GL_CCW);                        // Avant defini en conter-clock-wise (défaut)

        glColor3fv(color_tube);
        glBegin(GL_QUADS);
        for(i=0;i<12;i++)
        {
            glNormal3fv(vect_Zpos);
            glVertex3fv(tabQua2[i]);
        }
        glEnd();
        // Le corps du satellite
        glColor3fv(color_cover);
        glBegin(GL_TRIANGLE_FAN);
        for(i=0;i<8;i++)
        {
            glNormal3fv(normFan[i]);
            glVertex3fv(tabFan[i]);
        }
        glEnd();
        glBegin(GL_TRIANGLE_FAN);
        for(i=8;i<16;i++)
        {
            glNormal3fv(normFan[i]);
            glVertex3fv(tabFan[i]);
        }
        glEnd();
        glBegin(GL_QUADS);
        for(i=0;i<72;i++)
        {
            glNormal3fv(normQua[i]);
            glVertex3fv(tabQua[i]);
        }
        glEnd();
        // Plaque au fond du baffle
        glColor3fv(color_bluk);
        glBegin(GL_TRIANGLE_FAN);
        for(i=0;i<Nbaf+2;i++)
        {
            glNormal3fv(normtfb[i]);
            glVertex3fv(tabtfb[i]);
        }
        glEnd();
        // Plaque séparation
        glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE); // On ne traite faces avants et arrieres
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL) ; // Peint surfaces avants et arrieres
        glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE); // Mode de reflection
        glDisable(GL_CULL_FACE);                    // N'élimine pas la face arriere

        glColor3fv(color_cover);
        glBegin(GL_QUADS);
        for(i=0;i<24;i++)
        {
            glNormal3fv(vect_Zpos);
            glVertex3fv(tabqa3[i]);
        }
        glEnd();
        // Le baffle (cylindre de visee)
        glColor3fv(color_baff);
        glBegin(GL_QUADS);
        for(i=0;i<4*Nbaf;i++)
        {
            glNormal3fv(normbaf[i]);
            glVertex3fv(tabbaf[i]);
        }
        glEnd();
        // Mini-antenne
        glColor3fv(color_anten);
        glBegin(GL_QUADS);
            for(i=0;i<4*Npoly;i++)
            {
                glNormal3fv(norm_anten[i]);
                glVertex3fv(pt_anten[i]);
            }
        glEnd();
        // viewer emet sa lumiere
        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, color_red);
        glColor3fv(color_red);
        glBegin(GL_TRIANGLES);        
            glNormal3fv(norm_ray);
            for(i=0;i<3;i++)
            {
                glVertex3fv(pt_ray[i]);
            }
        glEnd();
        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, eteinte);
    }
    glEndList();
    return shape;
}
