#ifndef ORBITGEO_H
#define ORBITGEO_H

/**
   La classe OrbitGeo definit une orbite a partir des parametre orbitaux n, e, i, petit_om, grand_Om.
   La date de ces elements est egalement memorisee. Les autres elements sont calcules. 
   Les unites pour toutes les variables sont en S.I. L'origine des temps (en secondes) est
   le 1er janvier 2000 a midi a Greenwich (J2000.0).
   @author Michel Llibre - ONERA/DCSD.
 */

class OrbitGeo
{
public :
/* ---------------------------------------------------
  Variables de la classe : Unites SI si non precisees
 * ------------------------------------------------- */
double a0; // Demi grand axe (m)
double e0; // Excentricite
double i0; // Inclinaison (radians)
double ci0; // Cosinus inclinaison
double si0; // Sinus inclinaison
double  m3_HN[9]; // Matrice RotX(i0)
double om0; // Argument du perigee en radians (nul si circulaire)
double delta1; // Coef de correction du a J2 pour modele SGP
double omP; // d(arg_perigee)/dt en rd/s
double gdOm0; // Ascension droite du noeud ascendant en radians
double gdOmP; // d(Ascension_droite)/dt en rd/s
double n0; // Mouvement moyen en rd/s
double nnod; // Mouvement nodal (n0 + omP) en rd/s
double Tn0; // Periode mouvement moyen en rd/s
double Tnod;// Periode nodale en secondes
int Qrep; // Nombre de jours de la periode de repetitivite
int Krep; // Nombre de revolutions dans la periode de repetitivite

// Constructeurs
OrbitGeo(){}
/** Construction d'une orbite de satellite a partir de ses elements orbitaux
	 *  en unites S.I.
   @param n0		mouvement moyen(radians/s)
   @param e             excentricite
   @param i0		inclinaison de l'orbite (radians)
   @param om0		argument du perigee (radians)
   @param gdOm0		ascension droite du noeud ascendant (radians)
	 */	
OrbitGeo(double n0, double e, double i0, double om0, double Om0);

void set(double n0, double e, double i0, double om0, double Om0);

/* Impression des parametres orbitaux */
void print();
};
#endif

