#ifndef QTGLUTILS_H
#define QTGLUTILS_H
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>

/*
 * Chargement d'une texture depuis un fichier image
 * Entrée :
 *      filename  : fichier image à charger
 *      minfilter : Minification filter
 *      magfilter : Magnification filter
 *      wrap      : Repetition ou clamp
 * Sortie :
 *      GLuint de l'objet texture ou 0 si erreur
*/
GLuint loadTexture(char *filename, GLenum minfilter, GLenum magfilter, GLenum wrap);

/*
 *  Fabrication d'une DisplayListe openGL d'affichage d'une orbite
 *  Entrée :
 *     a : demi-grand axe de l'orbite
 *     e : excentricité de l'orbite
 *     I : inclinaison de l'orbite
 *     O : longitude du noeud ascendant de l'orbite
 *     w : argument du périgée
 *  Retour : numéro de la DisplayList générée.
 *
*/
GLuint newDlPanete(float rayon, int slices, char *nomfich);

/*
 *  Fabrication d'une DisplayListe openGL d'afficage d'une sphère
 *  Entrée :
 *     slices : Nombre de côtes d'orange de la planète
 *     rayon : rayon de la sphère
 *  Retour : numéro de la DisplayList générée.
 *
*/
GLuint newDlSphere(float rayon, int slices);

#endif // QTGLUTILS_H
