#include <math.h>
#include <stdio.h>
#include "angles.h"
// Conversion degres decimaux en HH MM SS.S et string
// Usage : str = deg2hms(buf, ang, &hh, &mhm, &ss);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en degre
// Sortie : str = HHhMMmSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *deg2hms(char *buffer, double ang, int *h, int *m, double *s)
{
double hh, mm;
long jours;
    hh = ang/15. + 5.e-9; // +1.8e-5 secondes de temps pour obliger à passer les caps.
	// On enlève les jours et on ramène entre 0 et 24h.
	if (hh < 0.) { hh = -hh; jours = (long) (hh/24.); hh = 24. - (hh - 24.*jours);}
	else { jours = (long) (hh/24.); hh = hh - 24.*jours;} 
	*h = (int) hh;
    mm = (hh - *h)*60.0; *m = (int) mm;
    *s = (mm - *m)*60.;
    sprintf(buffer,"%02dh%02dm%06.3lfs",*h,*m,*s);
	return buffer;	
}

// Conversion radians en HH MM SS.S et string
// Usage str = rad2hms(buf, ang, &h, &m, &s);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en radian
// Sortie : str = string HHhMMhSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999... 
char *rad2hms(char *buffer, double ang, int *h, int *m, double *s)
{
	return deg2hms(buffer, ang*180./M_PI, h, m, s);
}

// Conversion degres, minutes secondes en degres decimaux
// Usage : ang = dms2deg(d, m, s);
double dms2deg(int d, int m, double s)
{
	return d + (m+ s/60.)/60. ;
}

// Conversion heures, minutes secondes en heures decimales
// Usage : hours = hms2hours(h, m, s);
double hms2hours(int h, int m, double s)
{
	return h + (m+ s/60.)/60. ;
}
// Conversion degres, minutes secondes en radians
// Usage : ang = dms2rad(d, m, s);
double dms2rad(int d, int m, double s)
{
	return dms2deg(d, m, s)*M_PI/180. ;
}

// Conversion heures, minutes secondes en degres decimaux
// Usage : ang = hms2deg(h, m, s)
double hms2deg(int h, int m, double s)
{
	return 15.*hms2hours(h, m, s) ;
}

// Conversion heures, minutes secondes en jour décimal
// Usage : jours = hms2jours(h, m, s)
double hms2jours(int h, int m, double s)
{
	return hms2hours(h, m, s)/24.;
}

// Conversion heures, minutes secondes en radians
// Usage : ang = hms2rad(h, m, s)
double hms2rad(int h, int m, double s)
{
	return hms2hours(h, m, s)*M_PI/12. ;
}

// Conversion jour décimal en heure minute seconde
// Usage : str = jours2hms(buffer, jours, &h, &m, &s);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en radians
// Sortie : str = JJJjHHhMMmSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *jours2hms(char *buffer, double jours, int *j, int *h, int *m, double *s)
{
	*j = (int) floor(jours);
    return deg2hms(buffer + sprintf(buffer,"%dj", *j), (jours - *j)*360., h, m, s);
}

// Conversion radian vers Degrees MM SS.S
//  Usage str = rad2dms(buffer, ang, &d, &m, &s)
// Entree : buffer : buffer pour le string resultat
//			ang = angle en radians
// Sortie : str = DDD°MM'SS".Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *rad2dms(char *buffer, double ang, int *d, int *m, double *s)
{
double dd, mm;
long tours;
    dd = ang*180/M_PI + 1.e-8; // + 3.6e-5 sec d'arc pour passer les caps
	// On enlève les tours et on ramène entre 0 et 360.
    if (dd < 0.) { dd = -dd; tours = (long) (dd/360.); dd = 360. - (dd - 360.*tours);}
    else { tours = (long) (dd/24.); dd = dd - 360.*tours;}
    *d = (int) dd;
    mm = (dd - *d)*60.0; *m = (int) mm;
    *s = (mm - *m)*60.;
    sprintf(buffer,"%d°%02d'%05.2lfs\"",*d,*m,*s);
	return buffer;	
}
