#ifndef JULIEN_H
#define JULIEN_H

#ifdef __cplusplus
extern "C" {
#endif

    /** Cte : nombre de secondes par jour */
        extern const double Date_jour2s;

	/** Cte : valeur de J2000 (01/01/2000 a 12h) en jours julien */
	extern const double Date_J2000_j;

	/** Cte : Duree J2000 - 1/1/1970 0h en secondes */
	extern const double Date_Sys2J2000_s;

	/** Cte : valeur de J1900 (31/12/1899 a 12h) en jours julien */
	extern const double Date_J1900_j;
	
/**
 * Calcul ecart deltaET entre temps des ephemerides et temps universel
 * @param date 	date actuelle en jours julien
 * @return	l'ecart deltaET en jours (TE - TU)
 */
double jul2deltaET(double date);

/** Fonction convertissant un string date du type "2000:05:20 14:00:00" en jour julien
         @param sdate   Entree : le string date
         @return     La date en jour julien
 */
double str19tojul(char *sdate);

/** Fonction ajoutant à la date d'un string date du type "2000:05:20 14:00:00"
    un écart specifie en jour decimaux
         @param sdate   Entree : le string date
         @param ecart   Entree : l'ecart en jour decimaux
         @return     La date en jour julien (pointeur sur l'entree)
*/
char *add2date(char *sdate, double ecart);

/** Fonction convertissant les dates en jour julien
         @param j    Entree : jour décimal de 1.000 à 31.999 (1.5 : Le 1er à midi)
         @param m    Entree : mois de 1 à 12
         @param a	 Entree : année (attention 1 avant JC est noté 0, etc..)
         @return     La date en jour julien
 */
double dat2jul(double j, int m, int a);

/** Fonction donnant le jour de la semaine correspodant à un jour julien donné
         @param JJ    Entree :  date en jour julien
         @return 	  Le jour (Lundi, ...,Dimanche) de la semaine
         */
const char  * jul2jour(double JJ);

/** Fonction convertissant les jours juliens en date decimale
         @param jdec  Sortie : jdec[0] : jour décimal (1.5 = le 1er à midi)
         @param m	  Sortie : m[0] : mois (1 à 12)
         @param a	  Sortie : a[0] : année (attention 1 avant JC est noté 0, etc..)
         @param JJ    Entree :  date en jour julien
         @return 	  Le jour (Lundi, ...,Dimanche) de la semaine
         */
const char  * jul2dat(double *jdec, int *m, int *a, double JJ);

/** Fonction convertissant la date en string à la seconde près
         @param strdate  Sortie : La date formatée A/M/J à XXhXXmnXXs
         @param jdec  Entrée jdec : jour décimal (1.5 = le 1er à midi)
         @param m	  Entrée m : mois (1 à 12)
         @param a	  Entrée a : année (attention 1 avant JC est noté 0, etc..)
         @return      pointeur sur la date
         */
char *dat2str(char *strdate, double jdec, int m, int a);

/** Fonction convertissant la date en string au centième de seconde
         @param strdate Sortie : La date formatée A/M/J à XXhXXmnXXs
         @param jdec  Entrée jdec : jour décimal (1.5 = le 1er à midi)
         @param m	  Entrée m : mois (1 à 12)
         @param a	  Entrée a : année (attention 1 avant JC est noté 0, etc..)
         @return      pointeur sur la date
         */
char *dat2str_2(char *strdate, double jdec, int m, int a);

/** Fonction convertissant les jours juliens en string date
         * à la seconde près
         @param strdate Sortie : La date formatée "A/M/J a XXhXXmnXXs"
         @param JJ    Entree : date en jour julien
         @return      pointeur sur la date
         */
char *jul2str (char *strdate, double JJ);

/** Fonction convertissant les jours juliens en string date
         * au centième de seconde
         @param strdate Sortie : La date formatée "A/M/J a XXhXXmnXXs"
         @param JJ    Entree : date en jour julien
         @return      pointeur sur la date
         */
char *jul2str_2 (char *strdate, double JJ);

/** Fonction convertissant les jours juliens en tableau date
         * à la seconde près
         @param date  Sortie : int[6] année, mois, jour, hh, mn, s.
         @param JJ    Entree : date en jour julien
         @return      Le jour (Lundi, ...,Dimanche) de la semaine
         */
const char  * jul2tdat(int *date, double JJ);

/** Fonction convertissant un tableau date en jour julien
         @param date  Sortie : int[6] année, mois, jour, hh, mn, s.
         @return     (double) la date en jour julien
         */
double tdat2jul(int *date);

/* Routine de test */
void testjulien();

#ifdef __cplusplus
}
#endif

#endif // de #ifndef JULIEN_H
